package eu.stork.sp;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.Properties;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.struts2.interceptor.ServletRequestAware;
import org.apache.struts2.interceptor.ServletResponseAware;
import org.apache.struts2.views.velocity.components.SetDirective;

import com.opensymphony.xwork2.Action;
import com.opensymphony.xwork2.ActionSupport;

import eu.stork.peps.auth.commons.IPersonalAttributeList;
import eu.stork.peps.auth.commons.PEPSUtil;
import eu.stork.peps.auth.commons.PersonalAttribute;
import eu.stork.peps.auth.commons.PersonalAttributeList;
import eu.stork.peps.auth.commons.STORKAuthnRequest;
import eu.stork.peps.auth.engine.STORKSAMLEngine;
import eu.stork.peps.exceptions.STORKSAMLEngineException;


/**
 * =========================================================================================
 * @author Indra
 * This Action Generates a SAML Request with the data given by the user,
 * then sends it to the selected PEPS url
 * =========================================================================================
 */

public class InitAction extends ActionSupport implements ServletRequestAware, ServletResponseAware {
	
	private static final long serialVersionUID = 3660074009157921579L;
	private static Properties configs; 
	private static ArrayList<Country> countries;	
	private static ArrayList<PersonalAttribute> attributeList;
	private static String spId;
	private static String providerName;
	private static String qaa;
	private static String returnUrl;

	private String SAMLRequest;
	private String pepsUrl; 
	private String citizenCountry;
	private String euromap;



	/**
	 * =========================================================================================
	 * Fill the country data in the JSP that is shown to the user in order to fill the requested
	 * data to generate a SAML request
	 * =========================================================================================
	 */
	public String populate() {
		//Loading sp.properties
		try {
			configs = SPUtil.loadConfigs(Constants.SP_PROPERTIES);
		} catch (IOException e) {
			throw new ApplicationSpecificServiceException("Could not load configuration file ", e.getMessage());
		}

		countries = new ArrayList<Country> ();

		// Using the country list from the STORK Control File (xml)
		if ((Boolean.valueOf(configs.getProperty(Constants.SP_VERSIONCONTROL)).booleanValue())) {
			ManageSTORKInfo versioninfo=new ManageSTORKInfo(configs.getProperty(Constants.SP_ENVIRONMENT));
			List<Country> countrynames=versioninfo.processSTORKInfo(configs.getProperty(Constants.SP_VERSIONINFOFILE));
			for ( Iterator<Country> iter = countrynames.iterator(); iter.hasNext(); ) {
	    		Country c=(Country)iter.next();
				countries.add(c);	
	        }			
			setPepsUrl(versioninfo.getPepsURL());
		}
		// Using the country list from sp.properties
		else
		{
			int numCountries = Integer.parseInt(configs.getProperty(Constants.COUNTRY_NUMBER));
			for(int i=1;i<=numCountries;i++){
				Country country = new Country(configs.getProperty("country"+i+".name"),configs.getProperty("country"+i+".name"));
				countries.add(country);	
			}
			setPepsUrl(configs.getProperty(Constants.SPEPS_URL));
		}
		setEuromap(configs.getProperty(Constants.SP_EUROMAP));
		return "populate";
	}
	
	
	
	/**
	 * =========================================================================================
	 * Generates SAML request with the data given by the user and redirects to the PEPS url
	 * =========================================================================================
	 */
	public String execute(){

		//Creating STORK Authentication Request
		STORKAuthnRequest authnRequest = new STORKAuthnRequest();
		
		//Filling Authentication Request fields
		//========================================================================================
		authnRequest.setIssuer(configs.getProperty(Constants.SP_NAME));
		System.out.println(getPepsUrl());
		authnRequest.setDestination(getPepsUrl());
		authnRequest.setProviderName(configs.getProperty(Constants.SP_NAME));	
		authnRequest.setQaa(Integer.parseInt(configs.getProperty(Constants.SP_QAALEVEL)));
		authnRequest.setAssertionConsumerServiceURL(configs.getProperty(Constants.SP_RETURN));
		authnRequest.setSpSector(configs.getProperty(Constants.SP_SECTOR));
		authnRequest.setSpInstitution(configs.getProperty(Constants.SP_NAME));
		authnRequest.setSpApplication(configs.getProperty(Constants.SP_APLICATION));
		authnRequest.setSpCountry(configs.getProperty(Constants.SP_COUNTRY));
		authnRequest.setSPID(configs.getProperty(Constants.SP_NAME));

		//Including selected Country
		authnRequest.setCitizenCountryCode(citizenCountry);

		//Loading Stork attributes to request
		IPersonalAttributeList pAttList = new PersonalAttributeList();
		int nAttr = Integer.parseInt(configs.getProperty(Constants.ATTRIBUTE_NUMBER));
		for(int i=1; i<=nAttr; i++){
			PersonalAttribute att = new PersonalAttribute();
			//Attribute name
			att.setName(configs.getProperty("attribute"+ i +".name"));
			//Attribute mandatory?
			if("true".equals(configs.getProperty("attribute"+ i +".mandatory")))
				att.setIsRequired(true);
			else
				att.setIsRequired(false);
			//Attribute value?
			String value = configs.getProperty("attribute"+ i + ".value");
			if(value != null){
				ArrayList<String> aux= new ArrayList<String>();
				aux.add(value);
				att.setValue(aux);
			}
			pAttList.add(att);
		}	
		authnRequest.setPersonalAttributeList(pAttList);
		//========================================================================================
		
		
		//Getting STORKSAMLEngine object 
		//========================================================================================
		try{
			STORKSAMLEngine engine = STORKSAMLEngine.getInstance(Constants.SP_CONF);
			authnRequest = engine.generateSTORKAuthnRequest(authnRequest);
		}catch(STORKSAMLEngineException e){
			throw new ApplicationSpecificServiceException("Could not generate token for Saml Request", e.getErrorMessage());
		}	
		byte[] token = authnRequest.getTokenSaml();
		
		//Giving values to "SAMLRequest" and "pepsUrl" form fields
		//========================================================================================
		SAMLRequest = PEPSUtil.encodeSAMLToken(token);
		//Redirect Citizen to the PEPS
		//========================================================================================
		return Action.SUCCESS;
	}	
	
	
	
	
	

	public void setAttrList(ArrayList<PersonalAttribute> attributeList) {
		InitAction.attributeList = attributeList;
	}

	public ArrayList<PersonalAttribute> getAttributeList() {
		return attributeList;
	}

	public void setSAMLRequest(String samlToken) {
		this.SAMLRequest = samlToken;
	}

	public String getSAMLRequest() {
		return SAMLRequest;
	}	

	public String getQaa() {
		return qaa;
	}

	public void setQaa(String qaa) {
		InitAction.qaa = qaa;
	}	
	
	public String getSpId() {
		return spId;
	}

	public void setSpId(String spId) {
		InitAction.spId = spId;
	}	
	
	public String getProviderName() {
		return providerName;
	}

	public void setProviderName(String providerName) {
		InitAction.providerName = providerName;
	}	
	
	public String getCitizenCountry() {
		return citizenCountry;
	}

	public void setCitizenCountry(String citizenCountry) {
		this.citizenCountry = citizenCountry;
	}	
	

	public String getReturnUrl() {
		return returnUrl;
	}

	public void setReturnUrl(String returnUrl) {
		InitAction.returnUrl = returnUrl;
	}	
	
	public String getPepsUrl() {
		return pepsUrl;
	}

	public void setPepsUrl(String pepsUrl) {
		this.pepsUrl = pepsUrl;
	}
	
	public ArrayList<Country> getCountries() {
		return countries;
	}

	public void setCountries(ArrayList<Country> countries) {
		InitAction.countries = countries;
	}	
	
	public void setServletRequest(HttpServletRequest request) {
	}

	public void setServletResponse(HttpServletResponse response) {
	}
	public String getEuromap() {
		return euromap;
	}



	public void setEuromap(String euromap) {
		this.euromap = euromap;
	}
	
	
	

}
