package eu.stork.sp;

import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.Properties;
import java.util.regex.Pattern;

import javax.xml.bind.JAXBContext;
import javax.xml.bind.JAXBElement;
import javax.xml.bind.JAXBException;
import javax.xml.bind.Unmarshaller;

import eu.stork.info.schema.CountriesType;
import eu.stork.info.schema.CountryType;
import eu.stork.info.schema.EnvironmentType;
import eu.stork.info.schema.EnvironmentsType;
import eu.stork.info.schema.StorkVersionInfoType;

public class ManageSTORKInfo {

	private static Properties config;
	private static Properties configsig;

	final String CONFIGSIGNFILE="SamlEngineSignConfig.xml";
	private static String environment;
	private String pepsURL;
	

	public ManageSTORKInfo (String environment) {
		try {
			setConfig(SPUtil.loadConfigs(Constants.SP_PROPERTIES));
			setConfigsig(SPUtil.loadXMLConfigs(CONFIGSIGNFILE));
			setEnvironment(environment);
		} catch (IOException e) {
			throw new ApplicationSpecificServiceException("Could not load configuration file ", e.getMessage());
		}
    }
	
    @SuppressWarnings("unchecked")
	List<Country> processSTORKInfo(String file) {
    	if (getEnvironment()!=null) {
			File storkfile=new File(file);
			if (storkfile.exists()) {
				if (XMLValidation.validateSignature(storkfile, getConfigsig().getProperty("keystorePath"), getConfigsig().getProperty("keyStorePassword"))) {
					JAXBContext context;
					try {
						context = JAXBContext.newInstance("eu.stork.info.schema");
		    			Unmarshaller u = context.createUnmarshaller();
			    		JAXBElement<StorkVersionInfoType> storkinfo = (JAXBElement<StorkVersionInfoType>) u.unmarshal(storkfile);
			    		if ((storkinfo.getValue().getCountries()!=null)&&
			    				(storkinfo.getValue().getCountries().getCountry().size()>0)) {
			    			return filterCountry(storkinfo.getValue().getCountries());
			    		}else System.out.println("Incorrect Enviroment: "+file);
					} catch (JAXBException e) {
				    	System.out.println("Bad Format: "+file);
					}
				}
				else {

					System.out.println("XML signature not valid: "+storkfile.getAbsolutePath()); 
					throw new ApplicationSpecificServiceException("Please, contact with your Administrator:","XML signature not valid");			
				}
			}
			else {
				System.out.println("File not found: "+storkfile.getAbsolutePath());
				throw new ApplicationSpecificServiceException("Please, contact with your Administrator:","XML info file not found");			
			}
		}
		return null;
	}
	
    private List<Country> filterCountry(CountriesType countries) {
    	List<Country> validcountries = new ArrayList<Country>();
    	CountryType mycountry=findMyCountryInfo(countries);
    	try{
    	EnvironmentType mycountryenv=getCountryEnvironment(mycountry.getEnvironments());
    	if (mycountryenv!=null) {
    		setPepsURL(mycountryenv.getSPUrl());
    		List<String> spmandatoryattributes=getMandatoryAttributes();
        	List<String> spdisabledcountries=getDisabledCountries();
        	
        	Iterator<CountryType> iter = countries.getCountry().iterator();
        	while (iter.hasNext()) {
        		CountryType country=(CountryType) iter.next();
        		EnvironmentType countryenv=getCountryEnvironment(country.getEnvironments());
        		if (validCountry(countryenv,mycountryenv,spmandatoryattributes,spdisabledcountries)
        				// not in disabled countries list
        				//&&(!spdisabledcountries.contains(country.getID()))) {
        				) {
        			validcountries.add(new Country(country.getID(),country.getName()));
        		}
        	}
    	}
    	}
    	catch (Exception e) {
    		e.printStackTrace();
    	}
    	return validcountries; 
	}
	
	private EnvironmentType getCountryEnvironment(EnvironmentsType env) {
    	EnvironmentType env_out=null;
    	if (env!=null) {
    		if ((getEnvironment().equals("TEST"))&&(env.getTest()!=null)) {
    			env_out=env.getTest();
    		} else if ((getEnvironment().equals("PRE-PROD"))&&(env.getPreProd()!=null)) {
    			env_out=env.getPreProd();
    		} else if ((getEnvironment().equals("PROD"))&&(env.getProd()!=null)) {
    			env_out=env.getProd();
    		} 
    	}
    	return env_out;
	}

	private List<String> getDisabledCountries() {
		List<String> disabledcountries = new ArrayList<String>();
		for (int i=1;i<=Integer.valueOf(getConfig().getProperty("discountry.number"));i++) {
			disabledcountries.add(getConfig().getProperty("discountry"+i));	
		}
		return disabledcountries;
	}


	private List<String> getMandatoryAttributes() {
		List<String> mandatoryattributes = new ArrayList<String>();
		for (int i=1;i<=Integer.valueOf(getConfig().getProperty("attribute.number"));i++) {
			if (Boolean.valueOf(getConfig().getProperty("attribute"+i+".mandatory")).booleanValue()) {
				getConfig().getProperty("attribute"+i+".name");
				mandatoryattributes.add(getConfig().getProperty("attribute"+i+".name"));	
			}
		}

		return mandatoryattributes;
	}


	private CountryType findMyCountryInfo(CountriesType countries) {
    	Iterator<CountryType> iter = countries.getCountry().iterator();
    	while (iter.hasNext()) {
    		CountryType country=(CountryType) iter.next();
    		if (country.getID().equals(getConfig().getProperty("sp.country"))) {
    			return country;
    		}
    	}
		return null;
	}


	private boolean validCountry(EnvironmentType countryenv,EnvironmentType mycountryenv,List<String> spmandatoryattributes,List<String> spdisabledcountries) {
		if (
			// QAALevel
				(countryenv.getMaxQAA()>=Integer.valueOf(getConfig().getProperty("sp.qaalevel"))) &&				
			// mycountry.SAMLSpecVersion-accept<=colleagues.SAMLSpecVersion-send 
//				isVersionMinorOrEqual(mycountryenv.getSAMLSpecVersionAccept(),mycountryenv.getSAMLSpecVersionSend()) &&
			// mycountry.SAMLSpecVersion-send>=colleagues.SAMLSpecVersion-accept 
//				isVersionMinorOrEqual(countryenv.getSAMLSpecVersionAccept(),mycountryenv.getSAMLSpecVersionSend()) &&
			// mandatory attributes
				haveMandatoryAttributes(countryenv.getAttributes().getAttribute(),spmandatoryattributes)
		) return true;	
		else return false;
	}
	
	
    private boolean isVersionMinorOrEqual(String version1, String version2) {
    	String v1 = processVersion(version1,".",4);
    	String v2 = processVersion(version2,".",4);
    	if (v1.compareTo(v2)<=0) return true;
    	return false;	
	}

    public static String processVersion(String version, String separator, int maxDeep) {
    	String[] split = Pattern.compile(separator, Pattern.LITERAL).split(version);
        StringBuilder sb = new StringBuilder();
        for (String s : split) {
            sb.append(String.format("%" + maxDeep + 's', s));
        }
        return sb.toString();
    }
    

	private boolean haveMandatoryAttributes(List<String> attributes,List<String> spmandatoryattributes) {
		if (spmandatoryattributes!=null)
			if (attributes.containsAll(spmandatoryattributes)) return true;
		return false;
	}

	public static Properties getConfig() {
		return config;
	}


	public static void setConfig(Properties config) {
		ManageSTORKInfo.config = config;
	}
	
	public static String getEnvironment() {
		return environment;
	}


	public static void setEnvironment(String environment) {
		ManageSTORKInfo.environment = environment;
	}

	public String getPepsURL() {
		return pepsURL;
	}

	public void setPepsURL(String pepsURL) {
		this.pepsURL = pepsURL;
	}
	
	public static Properties getConfigsig() {
		return configsig;
	}

	public static void setConfigsig(Properties configsig) {
		ManageSTORKInfo.configsig = configsig;
	}

}
