package eu.stork.sp;

import java.util.ArrayList;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import org.apache.struts2.interceptor.ServletRequestAware;
import org.apache.struts2.interceptor.ServletResponseAware;
import com.opensymphony.xwork2.Action;
import com.opensymphony.xwork2.ActionSupport;
import eu.stork.peps.auth.commons.PEPSUtil;
import eu.stork.peps.auth.commons.PersonalAttribute;
import eu.stork.peps.auth.commons.IPersonalAttributeList;
import eu.stork.peps.auth.commons.STORKAuthnResponse;
import eu.stork.peps.auth.engine.STORKSAMLEngine;
import eu.stork.peps.exceptions.STORKSAMLEngineException;

/**
 * =========================================================================================
 * @author Ignacio Iigo & Joaqun Alcalde-Morao Jensen
 * This Action receives a SAML Response, shows it to the user and then validates it
 * getting the attributes values
 * =========================================================================================
 */


public class ReturnAction extends ActionSupport implements ServletRequestAware, ServletResponseAware {
	
	private static final long serialVersionUID = 3660074009157921579L;
	private String SAMLResponse;
	private HttpServletRequest request;
	private ArrayList<PersonalAttribute> attrList;



	/**
	 * =========================================================================================
	 * Validates the request and displays the value of the requested attributes
	 * =========================================================================================
	 */	
	public String execute(){
		
		//Decodes incoming SAML Response
		byte[] decSamlToken = PEPSUtil.decodeSAMLToken(SAMLResponse);

		//Get SAMLEngine instance
		STORKSAMLEngine engine = STORKSAMLEngine.getInstance(Constants.SP_CONF);		
	
		//Validating SAML Authentication response
		STORKAuthnResponse authnResponse = null;
		try {
			authnResponse = engine.validateSTORKAuthnResponse(decSamlToken, (String) request.getRemoteHost());				
		}catch(STORKSAMLEngineException e){			
			throw new ApplicationSpecificServiceException("Could not validate token for Saml Response", e.getErrorMessage());
		}			
		
		//Reading Authentication Response
		if(!authnResponse.isFail()){
			//Get attributes
			IPersonalAttributeList personalAttributeList = authnResponse.getPersonalAttributeList();			
			setAttrList(new ArrayList<PersonalAttribute>(personalAttributeList.values()));
			return Action.SUCCESS;
		}else{	
			throw new ApplicationSpecificServiceException("Saml Response is fail", authnResponse.getMessage());			
		}
	}
	
	
	
	
	public void setServletRequest(HttpServletRequest request) {
		this.request = request;
	}

	public void setServletResponse(HttpServletResponse response) {
	}
	
	public void setAttrList(ArrayList<PersonalAttribute> attrList) {
		this.attrList = attrList;
	}

	public ArrayList<PersonalAttribute> getAttrList() {
		return attrList;
	}	
	
	public String getSAMLResponse() {
		return SAMLResponse;
	}

	public void setSAMLResponse(String samlResponse) {
		this.SAMLResponse = samlResponse;
	}

	
}