// This work  is Open Source and licensed by the European Commission under the conditions of the Lesser General Public License (LGPL) v3 (http://www.gnu.org/licenses/lgpl.html); you may not use this file except in compliance with the License. Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the License for the specific governing permissions and limitations under the License.
package eu.stork.sp;

import java.io.ByteArrayInputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.security.Key;
import java.security.KeyException;
import java.security.KeyStore;
import java.security.KeyStoreException;
import java.security.NoSuchAlgorithmException;
import java.security.Provider;
import java.security.PublicKey;
import java.security.cert.Certificate;
import java.security.cert.CertificateException;
import java.security.cert.CertificateFactory;
import java.util.List;

import javax.xml.crypto.AlgorithmMethod;
import javax.xml.crypto.KeySelector;
import javax.xml.crypto.KeySelectorException;
import javax.xml.crypto.KeySelectorResult;
import javax.xml.crypto.MarshalException;
import javax.xml.crypto.XMLCryptoContext;
import javax.xml.crypto.XMLStructure;
import javax.xml.crypto.dsig.SignatureMethod;
import javax.xml.crypto.dsig.XMLSignature;
import javax.xml.crypto.dsig.XMLSignatureException;
import javax.xml.crypto.dsig.XMLSignatureFactory;
import javax.xml.crypto.dsig.dom.DOMValidateContext;
import javax.xml.crypto.dsig.keyinfo.KeyInfo;
import javax.xml.crypto.dsig.keyinfo.KeyValue;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import org.w3c.dom.Document;
import org.w3c.dom.NodeList;
import org.xml.sax.SAXException;


public class XMLValidation {


	public static boolean validateSignature(File xmlfile,String keystorepath,String keystorepass) {
		boolean coreValidity=false;		
		
    	if (validateCore(xmlfile)) {
    		if (validateSignCertificate(xmlfile,keystorepath,keystorepass)) {
    			return coreValidity=true;
    		}
    		else
    		{
    			System.out.println("File '"+xmlfile.getAbsolutePath()+"': Signed with an untrusted certificate");
    		}
    	}
    	else {
			System.out.println("File '"+xmlfile.getAbsolutePath()+"': Signature not valid");
    	}
		return coreValidity;
	}

	
	
	
	public static boolean validateSignCertificate(File xmlfile,String keystorepath,String keystorepass) {
		boolean signaturevalid=false;
		DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
		dbf.setNamespaceAware(true);
	    Document doc;

		try {
			doc = dbf.newDocumentBuilder().parse(xmlfile);
			NodeList nl = doc.getElementsByTagName("ds:X509Certificate");
			String certString=nl.item(0).getTextContent().trim();
			certString="-----BEGIN CERTIFICATE-----\n"+certString+"\n-----END CERTIFICATE-----";
			Certificate myCert = CertificateFactory.getInstance("X509").generateCertificate(
					new ByteArrayInputStream(certString.getBytes("UTF-8")));
			if (isInKeystore(myCert,keystorepath,keystorepass)) {
				signaturevalid=true;
			}
		} catch (FileNotFoundException e) {
		} catch (SAXException e) {
		} catch (IOException e) {
		} catch (ParserConfigurationException e) {
		} catch (CertificateException e) {
		}

		
		return signaturevalid;
	}
	
	
	
	private static boolean isInKeystore(Certificate certificate,String keystorepath,String keystorepass) {
		boolean into=false;

		KeyStore ks;
		try {
			ks = KeyStore.getInstance("JKS");
	       	ks.load(new FileInputStream(keystorepath), keystorepass.toCharArray());
			if (ks.getCertificateAlias(certificate)!=null) {
				into=true;
			}
		} catch (KeyStoreException e) {
		} catch (NoSuchAlgorithmException e) {
		} catch (CertificateException e) {
		} catch (FileNotFoundException e) {
		} catch (IOException e) {
		}

		return into;
	}



	public static boolean validateCore(File xmlfile) {
 
    	boolean corevalid=false;
		DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
		dbf.setNamespaceAware(true);
		Document doc;
		try {
			doc = dbf.newDocumentBuilder().parse(xmlfile);
			NodeList nl = doc.getElementsByTagNameNS(XMLSignature.XMLNS,"Signature");
			if (nl.getLength() == 0) {
			    System.out.println("Cannot find Signature element");
			}
			String providerName = System.getProperty("jsr105Provider","org.jcp.xml.dsig.internal.dom.XMLDSigRI");
			XMLSignatureFactory fac = XMLSignatureFactory.getInstance("DOM",(Provider) Class.forName(providerName).newInstance());
	
			for (int i = 0; i < nl.getLength(); i++) {
			    DOMValidateContext valContext = new DOMValidateContext(new KeyValueKeySelector(),nl.item(i));
			    XMLSignature signature = fac.unmarshalXMLSignature(valContext);
			    corevalid = signature.validate(valContext); 
			}
		} catch (FileNotFoundException e) {
		} catch (SAXException e) {
		} catch (IOException e) {
		} catch (ParserConfigurationException e) {
		} catch (InstantiationException e) {
		} catch (IllegalAccessException e) {
		} catch (ClassNotFoundException e) {
		} catch (MarshalException e) {
		} catch (XMLSignatureException e) {
		}
		return corevalid;
    }

    private static class KeyValueKeySelector extends KeySelector {
    	@SuppressWarnings("unchecked")
		public KeySelectorResult select(KeyInfo keyInfo,KeySelector.Purpose purpose,AlgorithmMethod method,XMLCryptoContext context)throws KeySelectorException {
            if (keyInfo == null) {
            	throw new KeySelectorException("Null KeyInfo object!");
            }
            SignatureMethod sm = (SignatureMethod) method;
            List list = keyInfo.getContent();

            for (int i = 0; i < list.size(); i++) {
            	XMLStructure xmlStructure = (XMLStructure) list.get(i);
            	if (xmlStructure instanceof KeyValue) {
                    PublicKey pk = null;
                    try {
                        pk = ((KeyValue)xmlStructure).getPublicKey();
                    } 
                    catch (KeyException ke) {
                        throw new KeySelectorException(ke);
                    }

                    // make sure algorithm is compatible with method
                    if (algEquals(sm.getAlgorithm(), pk.getAlgorithm())) {
                        return new SimpleKeySelectorResult(pk);
                    }
            	}
            }
            throw new KeySelectorException("No KeyValue element found!");
    	}

    	static boolean algEquals(String algURI, String algName) {
            if (algName.equalsIgnoreCase("DSA") &&algURI.equalsIgnoreCase(SignatureMethod.DSA_SHA1)) {
            	return true;
            } 
            else if (algName.equalsIgnoreCase("RSA") &&algURI.equalsIgnoreCase(SignatureMethod.RSA_SHA1)) {
            	return true;
            } 
            else {
            	return false;
            }
    	}
    }

    private static class SimpleKeySelectorResult implements KeySelectorResult {
    	private PublicKey pk;
    	SimpleKeySelectorResult(PublicKey pk) {
    		this.pk = pk;
    	}

    	public Key getKey() { return pk; }
    }
    
    public static String license (){
    	return "This work  is Open Source and licensed by the European Commission under the conditions of the Lesser General Public License (LGPL) v3 (http://www.gnu.org/licenses/lgpl.html); you may not use this file except in compliance with the License. Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on an 'AS IS' BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the License for the specific governing permissions and limitations under the License.";
    }   
    
}